/*
   This file includes weak (i.e. optional) functions to perform SoC related
   initialization. They are:
	1). __mem_pre_c_init():
		Executed before C language initialization to make memory
		ready so that program data can be initialized. An example
		is to initialize DRAM.
		Since this is called before C initialization, please
		use provided macros to avoid problems.
	2). __soc_init():
		Further SoC intialization. Called after C language
		initialization, so it is a typical C function.
*/

#include "nds32_intrinsic.h"
#if (defined(NDS32_BASELINE_V3M) || defined(NDS32_BASELINE_V3))
#include "nds32_isr.h"
#endif
#include "config.h"
#ifdef CONFIG_AG101P_16MB
#include "ag101_16mb.h"
#else
#include "ag101.h"
#endif

#if BURN
void __mem_pre_c_init()		//The function is weak (optional)
{
//Example of initializing (DRAM) memory controller for AG101 before first use
	/* Do this because we can't let it use stack(memory) */
	asm volatile (".global __mem_pre_c_init_begin");
	asm volatile ("__mem_pre_c_init_begin:");
	/* Memory bank 0 timing parameter register */
#ifdef CONFIG_AG101P_16MB
	NDS32_REG_WRITE(SMC_BASE + BANK0TPR_OFFSET, 0x000ff3ff);
#else
	NDS32_REG_WRITE(SMC_BASE + BANK0TPR_OFFSET, 0x00151151);
#endif
	/* PMU - PLL/DLL Control Register 0 - Enable DLL */
	NDS32_SET_BITS(PMUC_BASE + PLLDLLCR0_OFFSET, 0x00010000);
#ifdef AG101
	NDS32_SET_BITS(PMUC_BASE + PLLDLLCR0_OFFSET, 0x00e00000);
	NDS32_SET_BITS(PMUC_BASE + MFPSR_OFFSET, 0x00000010);
	NDS32_SET_BITS(PMUC_BASE + SHTCR_OFFSET, 0x0007f000);
#endif
	/* SDRAM controller - set timing parameter */
	NDS32_REG_WRITE(SDRAMC_BASE + PARM1_OFFSET, SDRAMC_PARM1_INIT);
	NDS32_REG_WRITE(SDRAMC_BASE + PARM2_OFFSET, SDRAMC_PARM2_INIT);
	/* SDRAM controller - set config register */
	NDS32_REG_WRITE(SDRAMC_BASE + CONFIG1_OFFSET, SDRAMC_CONFIG1_INIT);
	NDS32_REG_WRITE(SDRAMC_BASE + CONFIG2_OFFSET, SDRAMC_CONFIG2_INIT);

	NDS32_REG_WAIT4_BIT_OFF(SDRAMC_BASE + CONFIG2_OFFSET, 0x1c);
	NDS32_REG_WRITE(SDRAMC_BASE + CONFIG2_OFFSET, 0x00000004);
	NDS32_REG_WAIT4_BIT_OFF(SDRAMC_BASE + CONFIG2_OFFSET, 0xffffffff);
	NDS32_REG_WRITE(SDRAMC_BASE + CONFIG2_OFFSET, 0x00000000);
	NDS32_REG_WAIT4_BIT_OFF(SDRAMC_BASE + CONFIG2_OFFSET, 0xffffffff);
	/* remap */
#ifdef REMAP
	remap();
#endif
	asm volatile ("j __mem_pre_c_init_end");
}
#endif

void	__soc_init() 		//The function is weak (optional)
{
#if BURN
	uart_init();
#endif
}
