#include "config.h"
#include "nds32_intrinsic.h"

#define NDS32_PERFCTL_EN0   (1UL << 0)
#define NDS32_PERFCTL_EN1   (1UL << 1)
#define NDS32_PERFCTL_EN2   (1UL << 2)
#define NDS32_PERFCTL_IE0   (1UL << 3)
#define NDS32_PERFCTL_IE1   (1UL << 4)
#define NDS32_PERFCTL_IE2   (1UL << 5)
#define NDS32_PERFCTL_OVF0  (1UL << 6)
#define NDS32_PERFCTL_OVF1  (1UL << 7)
#define NDS32_PERFCTL_OVF2  (1UL << 8)
#define NDS32_PERFCTL_KS0   (1UL << 9)
#define NDS32_PERFCTL_KS1   (1UL << 10)
#define NDS32_PERFCTL_KS2   (1UL << 11)
#define NDS32_PERFCTL_KU0   (1UL << 12)
#define NDS32_PERFCTL_KU1   (1UL << 13)
#define NDS32_PERFCTL_KU2   (1UL << 14)
#define NDS32_PERFCTL_SEL0_C (0UL << 15)
#define NDS32_PERFCTL_SEL0_I (1UL << 15)
#define NDS32_PERFCTL_SEL1  (0x3f << 16)
#define NDS32_PERFCTL_SEL2  (0x3f << 22)

#define NDS32_ISET          (0x07 << 0)
#define NDS32_IWAY          (0x07 << 3)
#define NDS32_ISIZE         (0x07 << 6)
#define NDS32_DSET          (0x07 << 0)
#define NDS32_DWAY          (0x07 << 3)
#define NDS32_DSIZE         (0x07 << 6)

char *__itoa(int value, char *string, int radix)
{
	char tmp[33];
	char *tp = tmp;
	int i;
	unsigned v;
	int sign;
	char *sp;

	if (radix > 36 || radix <= 1) {
		puts("itoa error ....\n");
		return 0;
	}

	sign = (radix == 10 && value < 0);
	if (sign)
		v = -value;
	else
		v = (unsigned)value;
	while (v || tp == tmp) {
		i = v % radix;
		v = v / radix;
		if (i < 10)
			*tp++ = i + '0';
		else
			*tp++ = i + 'a' - 10;
	}

	sp = string;

	if (sign)
		*sp++ = '-';
	while (tp > tmp)
		*sp++ = *--tp;
	*sp = 0;
	return string;
}

void initValue(char *pValue, int count)
{
	int i;
	for (i = 0; i < count; i++)
		pValue[i] = '\0';
}

void printS(const char *msg, unsigned int value, int radix)
{
	char str_value[32];
	char *pValue;

	initValue(str_value, 32);
	pValue = __itoa(value, str_value, radix);
	puts(msg);
	puts((const char *)pValue);
	puts("\n");

}

void startPFM()
{
	unsigned int counter = 0, ctrl = 0, msc_cfg = 0;

	msc_cfg = __nds32__mfsr(NDS32_SR_MSC_CFG);
	if(!(msc_cfg & 0x4)) {
		puts("This CPU config doesn't have pfm to use.\n");
		return;
	}

	__nds32__mtsr(counter, NDS32_SR_PFMC0);
	__nds32__dsb();

	/* Enable performance counter0 , Enable interrupt for counter0 */
	/* Calculate Cycles for counter0 */
	ctrl |= NDS32_PERFCTL_EN0 | NDS32_PERFCTL_IE0 | NDS32_PERFCTL_SEL0_C;
	__nds32__mtsr(ctrl, NDS32_SR_PFM_CTL);
}

void executeInstructions()
{
	volatile int i, j;
	for (i = 0, j = 0; i < 2000000; i++)
		j++;
}

void stopPFM()
{
	unsigned int pfm_ctl, msc_cfg = 0;

	msc_cfg = __nds32__mfsr(NDS32_SR_MSC_CFG);
	if(!(msc_cfg & 0x4)) {
		puts("This CPU config doesn't have pfm to use.\n");
		return;
	}

	pfm_ctl = __nds32__mfsr(NDS32_SR_PFM_CTL);
	//get PFM_CTL
	pfm_ctl = pfm_ctl & 0xFFFFFFF8;
	__nds32__mtsr(pfm_ctl, NDS32_SR_PFM_CTL);

}

void readResult()
{

	unsigned int msc_cfg = 0;
	msc_cfg = __nds32__mfsr(NDS32_SR_MSC_CFG);
	if(!(msc_cfg & 0x4)) {
		puts("This CPU config doesn't have pfm to use.\n");
		return;
	}
	puts("Reading Performance Monitor Result\n");
	unsigned int counter = __nds32__mfsr(NDS32_SR_PFMC0);
	printS("The cycle count is ", counter, 10);
}

void enableIDCache()
{

	unsigned int cache_ctl, mmu_ctl, icm_cfg, dcm_cfg;
	unsigned int iset, iway, isize;
	unsigned int dset, dway, dsize;

	/* Get ICache ways, sets, line size  */
	icm_cfg = __nds32__mfsr(NDS32_SR_ICM_CFG);
	if ((icm_cfg & NDS32_ISET) < 7)
		iset = 1 << ((icm_cfg & NDS32_ISET) + 6);
	else
		iset = 0;
	printS("The ICache sets = ", iset, 10);

	iway = ((icm_cfg & NDS32_IWAY) >> 3) + 1;
	printS("The ICache ways = ", iway, 10);

	if (((icm_cfg & NDS32_ISIZE) >> 6)
	    && ((icm_cfg & NDS32_ISIZE) >> 6) <= 5)
		isize = 1 << (((icm_cfg & NDS32_ISIZE) >> 6) + 2);
	else if (((icm_cfg & NDS32_ISIZE) >> 6) >= 6) {
		puts("Warning i cacheline size is reserved value\n");
		isize = 0;
	} else
		isize = 0;
	printS("The ICache line size = ", isize, 10);
	if(isize == 0)
		puts("This CPU doesn't have ICache.\n");
	else {
		cache_ctl = __nds32__mfsr(NDS32_SR_CACHE_CTL);
		cache_ctl |= 0x1;
		/* Enable I Cache */
		__nds32__mtsr(cache_ctl, NDS32_SR_CACHE_CTL);
	}
	/* Get DCache ways, sets, line size  */
	dcm_cfg = __nds32__mfsr(NDS32_SR_DCM_CFG);
	if ((dcm_cfg & NDS32_DSET) < 7)
		dset = 1 << ((dcm_cfg & NDS32_DSET) + 6);
	else
		dset = 0;
	printS("The DCache sets = ", dset, 10);

	dway = ((dcm_cfg & NDS32_DWAY) >> 3) + 1;
	printS("The DCache ways = ", dway, 10);

	if (((dcm_cfg & NDS32_DSIZE) >> 6)
	    && ((dcm_cfg & NDS32_DSIZE) >> 6) <= 5)
		dsize = 1 << (((dcm_cfg & NDS32_DSIZE) >> 6) + 2);
	else if (((dcm_cfg & NDS32_DSIZE) >> 6) >= 6) {
		puts("Warning d cacheline size is reserved value\n");
		dsize = 0;
	} else
		dsize = 0;
	printS("The DCache line size = ", dsize, 10);
	if(dsize == 0)
		puts("This CPU doesn't have DCache.\n");
	else {
		cache_ctl = __nds32__mfsr(NDS32_SR_CACHE_CTL);
		cache_ctl |= 0x2;
		/* Enable D Cache */
		__nds32__mtsr(cache_ctl, NDS32_SR_CACHE_CTL);
	}

	/* Cacheable/Write-Back for NTC0 */
	mmu_ctl = __nds32__mfsr(NDS32_SR_MMU_CTL);
	mmu_ctl |= 0x2 << 1;
	__nds32__mtsr(mmu_ctl, NDS32_SR_MMU_CTL);
	if(cache_ctl & 0x1)
		puts("Enable I cache\n");
	if(cache_ctl & 0x2)
		puts("Enable D cache\n");
	if(!(cache_ctl & 0x3))
		puts("Can't enable I/D cache\n");
}

int main()
{
	puts("\nPerformance Monitor Sample code\n");

	startPFM();
	executeInstructions();
	stopPFM();
	readResult();
	enableIDCache();
	startPFM();
	executeInstructions();
	stopPFM();
	readResult();
	puts("Completed.\n");

	while (1) ;
}
